﻿using Mud.Helpers;
using Mud.Properties;
using System;
using System.Diagnostics;
using System.Drawing;
using System.Runtime.InteropServices;
using System.Threading;
using System.Windows.Forms;
using Timer = System.Windows.Forms.Timer;

namespace Mud
{
	public partial class FlashMessage : Form
	{
		public static int LastFFIXVPID = int.MinValue;
		public static FlashMessage LastMessage;
		private Timer ActivateTimer;
		private Timer CloseTimer;

		private FlashMessage(LogColors Level, string Message, int IntervalSeconds, MessageSize Size)
		{
			if (LastMessage != null) LastMessage.Close();
			Logging.Write(LogColors.INFO, $"{Logging.OVERLAY} Flashing Message: {0}", Message);
			InitializeComponent();
			float FontSize;
			switch (Size)
			{
				case MessageSize.TINY:
					FontSize = 8f;
					break;

				case MessageSize.SMALL:
					FontSize = 16f;
					break;

				case MessageSize.MEDIUM:
					FontSize = 32f;
					break;

				case MessageSize.LARGE:
					FontSize = 64f;
					break;

				case MessageSize.HUGE:
					FontSize = 128f;
					break;

				default:
					FontSize = 32f;
					break;
			}
			label1.Font = new Font(label1.Font.FontFamily, FontSize, label1.Font.Style);
			label1.Text = Message;
			label1.ForeColor = Color.FromArgb(Level.GetDark().A, Level.GetDark().R, Level.GetDark().G, Level.GetDark().B);
			Width = Screen.PrimaryScreen.WorkingArea.Width;
			Height = (int)Math.Floor(Screen.PrimaryScreen.WorkingArea.Height * 0.15);
			StartPosition = FormStartPosition.CenterScreen;
			TopMost = true;
			CloseTimer = new Timer();
			CloseTimer.Interval = IntervalSeconds * 1000;
			CloseTimer.Tick += CloseDialog;
			CloseTimer.Start();
			if (Settings.Default.UI_ACTIVATE_FFXIV)
			{
				uint pid;
				GetWindowThreadProcessId(GetForegroundWindow(), out pid);
				Process p = Process.GetProcessById((int)pid);
#if RB_CN
				if (p.ProcessName.Equals("ffxiv")) LastFFIXVPID = p.Id;
#else
				if (p.ProcessName.Equals("ffxiv_dx11")) LastFFIXVPID = p.Id;
#endif
				ActivateTimer = new Timer();
				ActivateTimer.Interval = 1;
				ActivateTimer.Tick += ActivateFFXIV;
				ActivateTimer.Start();
			}
			LastMessage = this;
			ShowDialog();
		}

		public enum MessageSize
		{
			TINY,
			SMALL,
			MEDIUM,
			LARGE,
			HUGE
		}

		#region Static Methods

		public static void ActivateFFXIV()
		{
			if (Settings.Default.UI_ACTIVATE_FFXIV && LastFFIXVPID != int.MinValue)
			{
				Process p = Process.GetProcessById(LastFFIXVPID);
				if (p != null)
				{
					Logging.Write(LogColors.INFO, $"{Logging.OVERLAY} Activating FFXIV: {0}", p.Id);
					SetForegroundWindow(p.MainWindowHandle);
				}
			}
		}

		public static void Flash(LogColors Level, string Message)
		{
			new Thread(() => new FlashMessage(Level, Message, 1, MessageSize.LARGE)).Start();
		}

		public static void Flash(LogColors Level, string Message, MessageSize Size)
		{
			new Thread(() => new FlashMessage(Level, Message, 1, Size)).Start();
		}

		#endregion Static Methods

		#region Custom Methods

		private void ActivateFFXIV(object sender, EventArgs e)
		{
			ActivateTimer.Stop();
			ActivateFFXIV();
		}

		private void CloseDialog(object sender, EventArgs e)
		{
			CloseTimer.Stop();
			Close();
		}

		#endregion Custom Methods

		#region DLL Imports

		[DllImport("user32.dll")]
		public static extern IntPtr GetWindowThreadProcessId(IntPtr hWnd, out uint ProcessId);

		[DllImport("user32.dll")]
		private static extern IntPtr GetForegroundWindow();

		[DllImport("user32.dll")]
		private static extern bool SetForegroundWindow(IntPtr hWnd);

		#endregion DLL Imports
	}
}