﻿namespace Mud.Helpers
{
	using System.Windows.Media;

	public class LogColors
	{
		#region Log Items Color

		public static LogColors DANGER = new LogColors(Color.FromRgb(255, 0, 0), Color.FromRgb(232, 0, 0));         // Red
		public static LogColors INFO = new LogColors(Color.FromRgb(209, 95, 238), Color.FromRgb(128, 0, 128));      // Purple
		public static LogColors PRIMARY = new LogColors(Color.FromRgb(0, 128, 255), Color.FromRgb(0, 0, 255));      // Blue
		public static LogColors SUCCESS = new LogColors(Color.FromRgb(0, 255, 0), Color.FromRgb(0, 153, 0));        // Green
		public static LogColors WARNING = new LogColors(Color.FromRgb(255, 128, 0), Color.FromRgb(204, 102, 0));    // Orange

		#endregion Log Items Color

		private Color dark;
		private System.Drawing.Color drawingDark;
		private System.Drawing.Color drawingLight;
		private Color light;

		public LogColors(Color light, Color dark)
		{
			SetLight(light);
			SetDark(dark);
			SetDrawingLight(System.Drawing.Color.FromArgb(GetLight().A, GetLight().R, GetLight().G, GetLight().B));
			SetDrawingDark(System.Drawing.Color.FromArgb(GetDark().A, GetDark().R, GetDark().G, GetDark().B));
		}

		public Color GetDark()
		{
			return dark;
		}

		public System.Drawing.Color GetDrawingDark()
		{
			return drawingDark;
		}

		public System.Drawing.Color GetDrawingLight()
		{
			return drawingLight;
		}

		public Color GetLight()
		{
			return light;
		}

		private void SetDark(Color value)
		{
			dark = value;
		}

		private void SetDrawingDark(System.Drawing.Color value)
		{
			drawingDark = value;
		}

		private void SetDrawingLight(System.Drawing.Color value)
		{
			drawingLight = value;
		}

		private void SetLight(Color value)
		{
			light = value;
		}
	}

	public class Logging
	{
		#region Log Items Name

		public const string BOT_INFO = "[Info]";
		public const string COMBAT = "[Combat]";
		public const string DATA_ERROR = "[Data Error]";
		public const string NAVIGATOR = "[Navigator]";
		public const string OVERLAY = "[Overlay]";
		public const string SETTINGS = "[Settings]";

		#endregion Log Items Name

		public static void Write(LogColors level, string message, params object[] args)
		{
			ff14bot.Helpers.Logging.Write(level.GetLight(), $"[MudAssist] {message}", args);
		}

		public static void Write(string message, params object[] args)
		{
			Write(LogColors.INFO, message, args);
		}
	}
}