﻿namespace Mud
{
	using System;
	using System.Collections.Generic;
	using System.Collections.Specialized;
	using System.Globalization;
	using System.Linq;
	using System.Threading;
	using System.Windows.Forms;
	using System.Windows.Input;
	using ff14bot;
	using ff14bot.AClasses;
	using ff14bot.Behavior;
	using ff14bot.Enums;
	using ff14bot.Managers;
	using ff14bot.Navigation;
	using ff14bot.Objects;
	using ff14bot.RemoteWindows;
	using Helpers;
	using Properties;
	using TreeSharp;
	using Action = TreeSharp.Action;

#if !RB_CN

	using ff14bot.Pathing.Service_Navigation;

#endif

	public class MudBase : BotBase
	{
		public static string LastTargetName;
		public const string Version = "2.0.8";

		#region Selectable Values

		public static string[] ModifierKeyStrings = { "None", "Shift", "Control", "Alt" };
		public static string[] TargetingModes = { "None", "Assist Tank", "Being Tanked", "Nearest Enemy" };
		public static string[] TargetListTypes = { "None", "Whitelist", "Blacklist" };
		public static string[] MovementModes = { "Combat", "Tank", "Follow" };
#if RB_CN
		public static string[] SupportedNavigationProviders = { "Null", "(CN only) Gaia" };
#else
		public static string[] SupportedNavigationProviders = { "Null", "(EU/US only) Service" };
#endif

		#endregion Selectable Values

		#region BotBase

		public bool TreeTick()
		{
			if (Core.Player.CurrentTarget != null && Core.Player.CurrentTarget.Name != null) LastTargetName = Core.Player.CurrentTarget.Name;
			WaypointManager.Track();
			SettingsForm.UpdateStatus();
			return true;
		}

		public static void UpdateNavigationProvider()
		{
			Logging.Write(LogColors.PRIMARY, $"{Logging.SETTINGS} -> Navigation updated to {0}", SupportedNavigationProviders[Settings.Default.SELECTED_NAVIGATION_PROVIDER]);
			switch (SupportedNavigationProviders[Settings.Default.SELECTED_NAVIGATION_PROVIDER])
			{
				case "Null":
					if (!(Navigator.NavigationProvider is NullProvider))
					{
						ServiceNavigationProvider g = (Navigator.NavigationProvider as ServiceNavigationProvider);
						if (g != null) g.Dispose();
						Navigator.NavigationProvider = new NullProvider();
						Logging.Write(LogColors.PRIMARY, $"{Logging.SETTINGS} -> Using Null Navigator");
					}
					break;

#if RB_CN
				case "(CN only) Gaia":
					if (!(Navigator.NavigationProvider is GaiaNavigator))
					{
						Navigator.NavigationProvider = new GaiaNavigator();
						Logging.Write(LogLevel.PRIMARY, $"{LogLevel.SETTINGS} -> Using Gaia Navigator");
					}
					break;
#else
				case "(EU/US only) Service":
					if (!(Navigator.NavigationProvider is ServiceNavigationProvider))
					{
						Navigator.NavigationProvider = new ServiceNavigationProvider();
						Logging.Write(LogColors.PRIMARY, $"{Logging.SETTINGS} -> Using Service Navigator");
					}
					break;
#endif

				default: break;
			}
		}

		#endregion BotBase

		#region Overrides

		public override string Name => "Mud Assist";
		public override bool WantButton => true;
		public override bool RequiresProfile => false;
		public override PulseFlags PulseFlags => PulseFlags.All;

		public override void OnButtonPress()
		{
			Logging.Write(LogColors.INFO, $"{Logging.SETTINGS} -> Opening settings form");
			SettingsForm settings = new SettingsForm();
			settings.ShowDialog();
		}

		public override void Initialize()
		{
			Logging.Write(LogColors.PRIMARY, $"{Logging.BOT_INFO} Initializing Mud Assist v" + Version);
			UnregisterAllHotkeys();
		}

		public override void Start()
		{
			UpdateNavigationProvider();
			Navigator.PlayerMover = new SlideMover();
			Logging.Write(LogColors.PRIMARY, $"{Logging.BOT_INFO} Started");
			ResetHotkeys();
		}

		public override void Stop()
		{
			_root = null;
			Navigator.PlayerMover = new NullMover();
#if RB_CN
			GaiaNavigator g = (Navigator.NavigationProvider as GaiaNavigator);
#else
			ServiceNavigationProvider g = (Navigator.NavigationProvider as ServiceNavigationProvider);
#endif
			if (g != null) g.Dispose();
			Navigator.NavigationProvider = new NullProvider();
			Logging.Write(LogColors.PRIMARY, $"{Logging.BOT_INFO} Stopped");
			UnregisterAllHotkeys();
		}

		private Composite _root;

		public override Composite Root => _root ?? (_root =
					new Decorator(
						req => TreeTick()
							&& !Settings.Default.COMBAT_ROUTINE_PAUSED
							&& Core.Player.IsAlive
							&& !Core.Player.IsCasting
							&& (!MovementManager.IsMoving
							|| Settings.Default.COMBAT_ROUTINE_EXECUTE_WHILE_MOVING),
						new PrioritySelector(
							// Questing Stuff
							new Decorator(
								req => Settings.Default.AUTO_SKIP_CUTSCENES
									&& QuestLogManager.InCutscene,
								new Action(a =>
								{
									SendKeys.SendWait(Keys.Escape.ToString());
									if (SelectYesno.IsOpen) SelectYesno.ClickYes();
								})),
							new Decorator(
								req => Settings.Default.AUTO_TALK_NPCS
									&& Talk.DialogOpen,
								new Action(a => { Talk.Next(); })),
							new Decorator(
								req => Settings.Default.AUTO_ACCEPT_QUESTS
									&& JournalAccept.IsOpen,
								new Action(a => { JournalAccept.Accept(); })),
							// Auto-Sprint
							new Decorator(
								req => Settings.Default.AUTO_SPRINT
									&& !IsValidEnemy(Core.Player.CurrentTarget)
									&& !Core.Player.InCombat
									&& ActionManager.IsSprintReady
									&& MovementManager.IsMoving
									&& !DutyManager.InInstance
									&& !Core.Player.IsMounted
									|| Core.Player.InCombat
									&& ActionManager.IsSprintReady
									&& MovementManager.IsMoving
									&& !Core.Player.IsMounted
									|| DutyManager.InInstance
									&& ActionManager.IsSprintReady
									&& MovementManager.IsMoving
									&& !Core.Player.IsMounted,
								new Action(a => { ActionManager.Sprint(); })),
							// Resting
							new Decorator(
								req => RoutineManager.Current.RestBehavior != null
									&& !Core.Player.InCombat
									&& Settings.Default.COMBAT_ROUTINE_REST
									&& !Core.Player.IsMounted
									&& ActionManager.IsSprintReady,
								RoutineManager.Current.RestBehavior),
							// Out Of Combat Healing
							new Decorator(
								req => RoutineManager.Current.HealBehavior != null
									&& (Core.Player.InCombat
									|| Settings.Default.COMBAT_ROUTINE_HEAL_OUT_OF_COMBAT)
									&& Settings.Default.COMBAT_ROUTINE_HEAL
									&& !Core.Player.IsMounted,
								RoutineManager.Current.HealBehavior),
							// Pre-Combat Buffs
							new Decorator(
								req => RoutineManager.Current.PreCombatBuffBehavior != null
									&& !Core.Player.InCombat
									&& Settings.Default.COMBAT_ROUTINE_PRECOMBATBUFF
									&& !Core.Player.IsMounted,
								RoutineManager.Current.PreCombatBuffBehavior),
							// Stop Moving If Moving & In Range Of Target
							new Decorator(
								req => Settings.Default.AUTO_MOVE
									//&& MovementManager.IsMoving
									&& WaypointManager.IsNavigating
									&& WaypointManager.Next == null,
								new Action(a => { WaypointManager.StopNavigating(); })),
							// Move To Target If Not In Range & Not On The Move
							new Decorator(
								req => Settings.Default.AUTO_MOVE
									&& !Core.Player.IsCasting
									&& WaypointManager.Next != null,
								new Action(a => { WaypointManager.MoveToNext(); })),
							// Find Suitable Target -- Lowest HP Tanked
							new Decorator(
								req => (PartyManager.IsInParty
									&& TargetingModes[Settings.Default.SELECTED_TARGETING_MODE].Equals("Being Tanked")
									&& GetVisiblePartyMembers().Where(pm => IsTank(pm)).Count() > 0
									&& (!Core.Player.HasTarget
									|| !Core.Player.CurrentTarget.CanAttack)),
								new Action(a =>
								{
									IEnumerable<GameObject> objs = GameObjectManager.GameObjects.Where(o => IsValidEnemy(o)
										&& (o as Character).InCombat
										&& (o as Character).CurrentTargetId == GetPartyTank().ObjectId);
									if (objs != null && objs.Count() > 0) objs.OrderBy(o => o.CurrentHealthPercent).First().Target();
								})),
							// Find Suitable Target -- Assist Tank
							new Decorator(
								req => (PartyManager.IsInParty
									&& TargetingModes[Settings.Default.SELECTED_TARGETING_MODE].Equals("Assist Tank")
									&& GetVisiblePartyMembers().Where(pm => IsTank(pm)).Count() > 0
									&& (!Core.Player.HasTarget
									|| !Core.Player.CurrentTarget.CanAttack)),
								new Action(a => Assist(GetVisiblePartyMembers().Where(pm => IsTank(pm)).First()))),
							// Find Suitable Target -- Nearest Enemy
							new Decorator(
								req => TargetingModes[Settings.Default.SELECTED_TARGETING_MODE].Equals("Nearest Enemy")
									&& !Core.Player.HasTarget,
								new Action(a =>
								{
									GameObject target = GetClosestEnemyByName(Settings.Default.MOBS_TO_TARGET);
									if (target != null) target.Target();
								})),
							// Pull Buff Behavior
							new Decorator(
								req => RoutineManager.Current.PullBuffBehavior != null
									&& !Core.Player.InCombat
									&& IsValidEnemy(Core.Player.CurrentTarget)
									&& Settings.Default.COMBAT_ROUTINE_PULL_BUFF
									&& !PartyManager.IsInParty,
								RoutineManager.Current.PullBuffBehavior),
							// Pull Behavior
							new Decorator(
								req => !Core.Player.InCombat
									&& IsValidEnemy(Core.Player.CurrentTarget)
									&& Settings.Default.COMBAT_ROUTINE_PULL
									&& (!PartyManager.IsInParty || IsTank(Core.Player))
									&& Core.Player.CurrentTarget.Location.Distance3D(Core.Player.Location) <= (RoutineManager.Current.PullRange + Core.Player.CurrentTarget.CombatReach + (float)Settings.Default.AUTO_MOVE_TARGET_RANGE),
								new PrioritySelector(RoutineManager.Current.PullBehavior, RoutineManager.Current.CombatBehavior)),
							// Executed In Combat
							new Decorator(
								req => !Core.Player.IsMounted
									&& InCombat,
								new PrioritySelector(
									// Combat Buffs
									new Decorator(
										req => RoutineManager.Current.CombatBuffBehavior != null
											&& Settings.Default.COMBAT_ROUTINE_COMBATBUFF
											&& !Core.Player.IsMounted,
										RoutineManager.Current.CombatBuffBehavior),
									// Combat Routine
									new Decorator(
										req => RoutineManager.Current.CombatBehavior != null
											&& Settings.Default.COMBAT_ROUTINE_COMBAT
											&& !Core.Player.IsMounted
											&& IsValidEnemy(Core.Player.CurrentTarget)
											&& Core.Player.CurrentTarget.Location.Distance3D(Core.Player.Location) <= (RoutineManager.Current.PullRange + Core.Player.CurrentTarget.CombatReach + (float)Settings.Default.AUTO_MOVE_TARGET_RANGE),
										RoutineManager.Current.CombatBehavior))))));

		#endregion Overrides

		#region Hotkeys

		private static Hotkey _hotkeyPause;
		private static Hotkey _hotkeyTargetMode;
		private static Hotkey _hotkeyToggleMovement;
		private static Hotkey _hotkeyAddTargetTargetList;
		private static Hotkey _hotkeyAddRemTargetTargetList;
		private static Hotkey _hotkeyToggleMovementMode;

		public static void ResetHotkeys()
		{
			UnregisterAllHotkeys();
			// Hotkey To Pause/Unpause
			Keys key;
			CultureInfo cinfo = Thread.CurrentThread.CurrentCulture;
			TextInfo text = cinfo.TextInfo;
			try { key = (Keys)(new KeysConverter()).ConvertFromString(text.ToTitleCase(Settings.Default.HOTKEY_PAUSE)); }
			catch (Exception e) { key = Keys.None; }
			if (key != Keys.None)
			{
				_hotkeyPause = HotkeyManager.Register(
					"HK_MUD_PAUSE",
					key,
					(ModifierKeys)Enum.Parse(typeof(ModifierKeys),
					ModifierKeyStrings[Settings.Default.HOTKEY_MODIFIER_PAUSE]),
					hkPause =>
					{
						Settings.Default.COMBAT_ROUTINE_PAUSED = !Settings.Default.COMBAT_ROUTINE_PAUSED;
						if (Settings.Default.COMBAT_ROUTINE_PAUSED)
						{
							Logging.Write(LogColors.PRIMARY, $"{Logging.BOT_INFO} Paused");
							if (Settings.Default.UI_FLASH_MESSAGES) FlashMessage.Flash(LogColors.DANGER, "Paused");
						}
						else
						{
							Logging.Write(LogColors.PRIMARY, $"{Logging.BOT_INFO} Unpaused");
							if (Settings.Default.UI_FLASH_MESSAGES) FlashMessage.Flash(LogColors.SUCCESS, "Unpaused");
						}
					});
				Logging.Write(LogColors.INFO, $"{Logging.SETTINGS} Added Hotkey -> " + _hotkeyPause);
			}

			// Hotkey To Change Targeting Mode
			try { key = (Keys)(new KeysConverter()).ConvertFromString(text.ToTitleCase(Settings.Default.HOTKEY_TARGET_MODE)); }
			catch (Exception e) { key = Keys.None; }
			if (key != Keys.None)
			{
				_hotkeyTargetMode = HotkeyManager.Register(
					"HK_MUD_TARGET",
					key,
					(ModifierKeys)Enum.Parse(typeof(ModifierKeys),
					ModifierKeyStrings[Settings.Default.HOTKEY_MODIFIER_TARGET_MODE]),
					hkTargetMode =>
					{
						Logging.Write(LogColors.PRIMARY, $"{Logging.SETTINGS} Previous Targeting Mode -> " + TargetingModes[Settings.Default.SELECTED_TARGETING_MODE]);
						if ((Settings.Default.SELECTED_TARGETING_MODE + 1) == TargetingModes.Length) Settings.Default.SELECTED_TARGETING_MODE = 0;
						else Settings.Default.SELECTED_TARGETING_MODE = (Settings.Default.SELECTED_TARGETING_MODE + 1);
						SettingsForm.SelectTargetingMode(Settings.Default.SELECTED_TARGETING_MODE);
						Logging.Write(LogColors.PRIMARY, $"{Logging.SETTINGS} Target Mode -> " + TargetingModes[Settings.Default.SELECTED_TARGETING_MODE]);
						if (Settings.Default.UI_FLASH_MESSAGES) FlashMessage.Flash(LogColors.WARNING, "Target: " + TargetingModes[Settings.Default.SELECTED_TARGETING_MODE]);
					});
				Logging.Write(LogColors.INFO, $"{Logging.SETTINGS} Added Hotkey -> " + _hotkeyTargetMode);
			}

			// Hotkey To Toggle Movement
			try { key = (Keys)(new KeysConverter()).ConvertFromString(text.ToTitleCase(Settings.Default.HOTKEY_TOGGLE_MOVEMENT)); }
			catch (Exception e) { key = Keys.None; }
			if (key != Keys.None)
			{
				_hotkeyToggleMovement = HotkeyManager.Register(
					"HK_MUD_TOGGLE_MOVEMENT",
					key,
					(ModifierKeys)Enum.Parse(typeof(ModifierKeys),
					ModifierKeyStrings[Settings.Default.HOTKEY_MODIFIER_TOGGLE_MOVEMENT]),
					hkToggleMovement =>
					{
						Settings.Default.AUTO_MOVE = !Settings.Default.AUTO_MOVE;
						WaypointManager.StopNavigating();
						if (Settings.Default.AUTO_MOVE)
						{
							Logging.Write(LogColors.PRIMARY, $"{Logging.BOT_INFO} Movement ON");
							if (Settings.Default.UI_FLASH_MESSAGES) FlashMessage.Flash(LogColors.PRIMARY, "Movement ON");
						}
						else
						{
							Logging.Write(LogColors.DANGER, $"{Logging.BOT_INFO} Movement OFF");
							if (Settings.Default.UI_FLASH_MESSAGES) FlashMessage.Flash(LogColors.WARNING, "Movement OFF");
						}
					});
				Logging.Write(LogColors.INFO, $"{Logging.SETTINGS} Added Hotkey -> " + _hotkeyToggleMovement);
			}

			// Hotkey To Toggle Movement
			try { key = (Keys)(new KeysConverter()).ConvertFromString(text.ToTitleCase(Settings.Default.HOTKEY_MOVEMENT_MODE)); }
			catch (Exception e) { key = Keys.None; }
			if (key != Keys.None)
			{
				_hotkeyToggleMovementMode = HotkeyManager.Register(
					"HK_MUD_TOGGLE_MOVEMENT_MODE",
					key,
					(ModifierKeys)Enum.Parse(typeof(ModifierKeys),
					ModifierKeyStrings[Settings.Default.HOTKEY_MODIFIER_MOVEMENT_MODE]),
					hkToggleMovementMode =>
					{
						Logging.Write(LogColors.PRIMARY, $"{Logging.SETTINGS} Previous Movement Mode -> " + TargetingModes[Settings.Default.SELECTED_MOVEMENT_MODE]);
						if ((Settings.Default.SELECTED_MOVEMENT_MODE + 1) == MovementModes.Length) Settings.Default.SELECTED_MOVEMENT_MODE = 0;
						else Settings.Default.SELECTED_MOVEMENT_MODE = (Settings.Default.SELECTED_MOVEMENT_MODE + 1);
						SettingsForm.SelectMovementMode(Settings.Default.SELECTED_MOVEMENT_MODE);
						Logging.Write(LogColors.PRIMARY, $"{Logging.SETTINGS} Move Mode -> " + MovementModes[Settings.Default.SELECTED_MOVEMENT_MODE]);
						if (Settings.Default.UI_FLASH_MESSAGES) FlashMessage.Flash(LogColors.WARNING, "Move Mode: " + MovementModes[Settings.Default.SELECTED_MOVEMENT_MODE]);
					});
				Logging.Write(LogColors.INFO, $"{Logging.SETTINGS} Added Hotkey -> " + _hotkeyToggleMovementMode);
			}

			// Hotkey To Add/Remove Target From Target List
			try { key = (Keys)(new KeysConverter()).ConvertFromString(text.ToTitleCase(Settings.Default.HOTKEY_ADD_REM_TARGET_LIST)); }
			catch (Exception e) { key = Keys.None; }
			if (key != Keys.None)
			{
				_hotkeyAddRemTargetTargetList = HotkeyManager.Register(
					"HK_MUD_ADD_REM_TARGET",
					key,
					(ModifierKeys)Enum.Parse(typeof(ModifierKeys),
						ModifierKeyStrings[Settings.Default.HOTKEY_MODIFIER_REM_TARGET_LIST]),
					hkAddRemTargetTargetList =>
					{
						if (LastTargetName == null)
						{
							if (Settings.Default.UI_FLASH_MESSAGES) FlashMessage.Flash(LogColors.WARNING, "Last Target not valid", FlashMessage.MessageSize.LARGE);
						}
						else if (Settings.Default.MOBS_TO_TARGET.Contains(LastTargetName))
						{
							Settings.Default.MOBS_TO_TARGET.Remove(LastTargetName);
							SettingsForm.UpdateTargetDataGrid();
							Logging.Write(LogColors.INFO, $"{Logging.COMBAT} Target List -> Removing {0}", LastTargetName);
							if (Settings.Default.UI_FLASH_MESSAGES) FlashMessage.Flash(LogColors.PRIMARY, "Removing Target: " + LastTargetName, FlashMessage.MessageSize.LARGE);
						}
						else
						{
							Settings.Default.MOBS_TO_TARGET.Add(LastTargetName);
							SettingsForm.UpdateTargetDataGrid();
							Logging.Write(LogColors.INFO, $"{Logging.COMBAT} Target List -> Adding {0}", LastTargetName);
							if (Settings.Default.UI_FLASH_MESSAGES) FlashMessage.Flash(LogColors.WARNING, "Adding Target: " + LastTargetName, FlashMessage.MessageSize.LARGE);
						}
					});
				Logging.Write(LogColors.INFO, $"{Logging.SETTINGS} Added Hotkey -> " + _hotkeyAddRemTargetTargetList);
			}
		}

		public static void UnregisterAllHotkeys()
		{
			UnregisterHotkey(_hotkeyPause);
			UnregisterHotkey(_hotkeyTargetMode);
			UnregisterHotkey(_hotkeyToggleMovement);
			UnregisterHotkey(_hotkeyAddTargetTargetList);
			UnregisterHotkey(_hotkeyAddRemTargetTargetList);
			UnregisterHotkey(_hotkeyToggleMovementMode);
		}

		public static void UnregisterHotkey(Hotkey hk)
		{
			if (hk != null)
			{
				HotkeyManager.Unregister(hk);
				Logging.Write(LogColors.WARNING, $"{Logging.SETTINGS} Unregistered Hotkey -> " + hk);
			}
		}

		#endregion Hotkeys

		#region Helper Methods

		public static GameObject GetClosestEnemyByName(StringCollection names)
		{
			Logging.Write(LogColors.INFO, $"{Logging.COMBAT} Finding nearest enemy to attack...");
			return GameObjectManager.GameObjects.Where(u =>
				IsValidEnemy(u)
				&& (decimal)Core.Player.Location.Distance3D(u.Location) <= Settings.Default.MAX_TARGET_DISTANCE)
				.OrderBy(u => Core.Player.Location.Distance3D(u.Location)).FirstOrDefault();
		}

		public static List<Character> GetVisiblePartyMembers()
		{
			List<Character> members = new List<Character>();
			if (!PartyManager.IsInParty)
			{
				members.Add(Core.Player);
			}
			else
			{
				foreach (PartyMember pm in PartyManager.AllMembers)
				{
					if (pm.IsInObjectManager) members.Add((Character)GameObjectManager.GetObjectByObjectId(pm.ObjectId));
				}
			}

			return members;
		}

		public static bool IsTank(Character c)
		{
			switch (c.CurrentJob)
			{
				case ClassJobType.Marauder:
				case ClassJobType.Warrior:
				case ClassJobType.Paladin:
				case ClassJobType.Gladiator:
				case ClassJobType.DarkKnight:
					return true;

				default:
					return false;
			}
		}

		public static void Assist(Character c)
		{
			GameObject target = GameObjectManager.GetObjectByObjectId(c.CurrentTargetId);
			if (target != null && target.IsTargetable && target.IsValid && target.CanAttack)
			{
				Logging.Write(LogColors.INFO, $"{Logging.COMBAT} Assisting " + c.Name);
				target.Target();
			}
		}

		public static bool IsValidEnemy(GameObject obj)
		{
			if (obj == null) return false;
			if (!(obj is Character)) return false;
			Character c = (Character)obj;
			return !c.IsMe && !c.IsDead && c.IsValid && c.IsTargetable && c.IsVisible && c.CanAttack
				&& (TargetListTypes[Settings.Default.SELECTED_TARGET_LIST_TYPE].Equals("None")
				|| (TargetListTypes[Settings.Default.SELECTED_TARGET_LIST_TYPE].Equals("Whitelist")
				&& Settings.Default.MOBS_TO_TARGET.Contains(c.Name))
				|| (TargetListTypes[Settings.Default.SELECTED_TARGET_LIST_TYPE].Equals("Blacklist")
				&& !Settings.Default.MOBS_TO_TARGET.Contains(c.Name)));
		}

		public static Character GetPartyTank()
		{
			if (GetVisiblePartyMembers().Count > 0)
			{
				try { return GetVisiblePartyMembers().First(p => IsTank(p)); }
				catch (Exception e) { return null; }
			}

			return null;
		}

		public static bool InCombat => GetVisiblePartyMembers().Where(p =>
										  p.IsAlive
											  && p.HasTarget
											  && p.InCombat
											  && p.TargetCharacter != null
											  && p.TargetCharacter.HasTarget
											  && p.TargetCharacter.CurrentHealth < p.TargetCharacter.MaxHealth
		) != null;

		#endregion Helper Methods
	}
}