﻿namespace Mud.Properties
{
	using Helpers;
	using System.ComponentModel;
	using System.Configuration;

	// This class allows you to handle specific events on the settings class:
	// - The SettingChanging event is raised before a setting's value is changed.
	// - The PropertyChanged event is raised after a setting's value is changed.
	// - The SettingsLoaded event is raised after the setting values are loaded.
	// - The SettingsSaving event is raised before the setting values are saved.
	public sealed partial class Settings
	{
		public Settings()
		{
			// To add event handlers for saving and changing settings, uncomment the lines below:
			SettingChanging += SettingChangingEventHandler;
			SettingsSaving += SettingsSavingEventHandler;
		}

		private void SettingChangingEventHandler(object sender, SettingChangingEventArgs e)
		{
			Logging.Write(LogColors.DANGER, "[Settings] Changed [{0}] -> {1}", e.SettingName, e.NewValue);
		}

		private void SettingsSavingEventHandler(object sender, CancelEventArgs e)
		{
			Logging.Write(LogColors.DANGER, "[Settings] Saved");
			foreach (SettingsProperty p in Default.Properties)
			{
				Logging.Write(LogColors.DANGER, "[{0}] -> {1}", p.Name, p.DefaultValue.ToString());
			}
		}
	}
}