﻿namespace Mud
{
	using ff14bot.Managers;
	using Helpers;
	using Properties;
	using System;
	using System.ComponentModel;
	using System.Windows.Forms;

	public partial class SettingsForm : Form
	{
		private static SettingsForm Instance;

		public SettingsForm()
		{
			InitializeComponent();
			Instance = this;
			TopMost = Settings.Default.UI_ALWAYS_ON_TOP;
			Text = $"Mud Assist v{MudBase.Version} - Settings";
			dgvTargetList.DataSource = new BindingList<StringValue>();
			DataGridViewColumn column = dgvTargetList.Columns[0];
			column.Width = dgvTargetList.Width - 65;
			dgvTargetList.DataError += OnDataErrorTargetList;
			ReloadSettings();
			UpdateStatus();
		}

		#region Custom Methods

		public static void SelectMovementMode(int p)
		{
			if (Instance != null) Instance.cmbMovementMode.SelectedIndex = p;
		}

		public static void SelectTargetingMode(int p)
		{
			if (Instance != null) Instance.cmbTargetingMode.SelectedIndex = p;
		}

		public static void UpdateStatus()
		{
			if (Instance != null)
			{
				if (Settings.Default.COMBAT_ROUTINE_PAUSED)
				{
					Instance.tspPauseStatus.Text = "STOPPED";
					Instance.tspPauseStatus.ForeColor = LogColors.DANGER.GetDrawingDark();
				}
				else
				{
					Instance.tspPauseStatus.Text = "RUNNING";
					Instance.tspPauseStatus.ForeColor = LogColors.SUCCESS.GetDrawingDark();
				}
				if (Settings.Default.AUTO_MOVE)
				{
					Instance.tspMovementStatus.Text = "+AMOVE";
					Instance.tspMovementStatus.ForeColor = LogColors.PRIMARY.GetDrawingDark();
				}
				else
				{
					Instance.tspMovementStatus.Text = "-AMOVE";
					Instance.tspMovementStatus.ForeColor = LogColors.WARNING.GetDrawingDark();
				}
				Instance.tspFollowModeStatus.Text = "M: " + MudBase.MovementModes[Settings.Default.SELECTED_MOVEMENT_MODE].ToUpper();
				Instance.tspTargetModeStatus.Text = "T: " + MudBase.TargetingModes[Settings.Default.SELECTED_TARGETING_MODE].ToUpper();
			}
		}

		public static void UpdateTargetDataGrid()
		{
			if (Instance != null && Instance.dgvTargetList != null && Instance.dgvTargetList.DataSource != null)
			{
				BindingList<StringValue> targets = (BindingList<StringValue>)Instance.dgvTargetList.DataSource;
				targets.Clear();
				foreach (string s in Settings.Default.MOBS_TO_TARGET)
				{
					targets.Add(new StringValue(s));
				}
				Instance.dgvTargetList.Refresh();
			}
		}

		private void ReloadSettings()
		{
			cmbHotkeyModifierPause.Items.Clear();
			cmbHotkeyModifierPause.Items.AddRange(MudBase.ModifierKeyStrings);
			cmbHotkeyModifierPause.SelectedIndex = Settings.Default.HOTKEY_MODIFIER_PAUSE;
			cmbHotkeyModifierTargetMode.Items.Clear();
			cmbHotkeyModifierTargetMode.Items.AddRange(MudBase.ModifierKeyStrings);
			cmbHotkeyModifierTargetMode.SelectedIndex = Settings.Default.HOTKEY_MODIFIER_TARGET_MODE;
			cmbTargetingMode.Items.Clear();
			cmbTargetingMode.Items.AddRange(MudBase.TargetingModes);
			SelectTargetingMode(Settings.Default.SELECTED_TARGETING_MODE);
			cmbTargetListType.Items.Clear();
			cmbTargetListType.Items.AddRange(MudBase.TargetListTypes);
			cmbTargetListType.SelectedIndex = Settings.Default.SELECTED_TARGET_LIST_TYPE;
			cmbMovementMode.Items.Clear();
			cmbMovementMode.Items.AddRange(MudBase.MovementModes);
			cmbMovementMode.SelectedIndex = Settings.Default.SELECTED_MOVEMENT_MODE;
			cmbHotkeyModifierMovement.Items.Clear();
			cmbHotkeyModifierMovement.Items.AddRange(MudBase.ModifierKeyStrings);
			cmbHotkeyModifierMovement.SelectedIndex = Settings.Default.HOTKEY_MODIFIER_TOGGLE_MOVEMENT;
			cmbHotkeyModifierAddRemTargetList.Items.Clear();
			cmbHotkeyModifierAddRemTargetList.Items.AddRange(MudBase.ModifierKeyStrings);
			cmbHotkeyModifierAddRemTargetList.SelectedIndex = Settings.Default.HOTKEY_MODIFIER_REM_TARGET_LIST;
			cmbHotkeyModifierToggleMoveMode.Items.Clear();
			cmbHotkeyModifierToggleMoveMode.Items.AddRange(MudBase.ModifierKeyStrings);
			cmbHotkeyModifierToggleMoveMode.SelectedIndex = Settings.Default.HOTKEY_MODIFIER_MOVEMENT_MODE;
			cbxAutoFace.Checked = GameSettingsManager.FaceTargetOnAction;
			cmbNavigationProvider.Items.Clear();
			cmbNavigationProvider.Items.AddRange(MudBase.SupportedNavigationProviders);
			cmbNavigationProvider.SelectedIndex = Settings.Default.SELECTED_NAVIGATION_PROVIDER;
			cmbCombatRoutines.Items.Clear();
			RoutineManager.Routines.ForEach(r => cmbCombatRoutines.Items.Add(r.Name));
			cmbCombatRoutines.SelectedIndex = cmbCombatRoutines.Items.IndexOf(RoutineManager.Current.Name);
			UpdateTargetDataGrid();
		}

		private void UpdateTargetSettings(object sender, EventArgs e)
		{
			if (dgvTargetList != null && dgvTargetList.DataSource != null && dgvTargetList.RowCount > 0)
			{
				Settings.Default.MOBS_TO_TARGET.Clear();
				foreach (StringValue sv in (BindingList<StringValue>)dgvTargetList.DataSource)
				{
					Settings.Default.MOBS_TO_TARGET.Add(sv.Name);
				}
			}
		}

		#endregion Custom Methods

		#region Misc Events

		private void OnDataErrorTargetList(object sender, DataGridViewDataErrorEventArgs e)
		{
			Logging.Write(LogColors.WARNING, $"{Logging.DATA_ERROR} Try again if target was not successfully added/removed.");
		}

		#endregion Misc Events

		#region Button Events

		private void OnClickAddTargetList(object sender, EventArgs e)
		{
			if (MudBase.LastTargetName != null && !Settings.Default.MOBS_TO_TARGET.Contains(MudBase.LastTargetName))
			{
				Settings.Default.MOBS_TO_TARGET.Add(MudBase.LastTargetName);
				Logging.Write(LogColors.INFO, $"{Logging.COMBAT} Target List -> Adding {0}", MudBase.LastTargetName);
				UpdateTargetDataGrid();
				FlashMessage.ActivateFFXIV();
			}
		}

		private void OnClickRemoveTargetList(object sender, EventArgs e)
		{
			foreach (DataGridViewRow row in dgvTargetList.SelectedRows)
			{
				if (row != dgvTargetList.Rows[dgvTargetList.Rows.Count - 1]) Settings.Default.MOBS_TO_TARGET.Remove(((StringValue)row.DataBoundItem).Name);
			}
			Logging.Write(LogColors.INFO, $"{Logging.COMBAT} Target List -> Removing Mob");
			UpdateTargetDataGrid();
			FlashMessage.ActivateFFXIV();
		}

		private void OnClickReset(object sender, EventArgs e)
		{
			Logging.Write(LogColors.DANGER, $"{Logging.SETTINGS} Reset to Default");
			Settings.Default.Reset();
			ReloadSettings();
		}

		private void OnClickSave(object sender, EventArgs e)
		{
			UpdateTargetSettings(sender, e);
			Settings.Default.Save();
			MudBase.ResetHotkeys();
			Logging.Write(LogColors.PRIMARY, $"{Logging.SETTINGS} Saved");
		}

		private void OnClickSaveAndClose(object sender, EventArgs e)
		{
			UpdateTargetSettings(sender, e);
			Settings.Default.Save();
			MudBase.ResetHotkeys();
			Logging.Write(LogColors.PRIMARY, $"{Logging.SETTINGS} Saved and Closed");
			Close();
		}

		#endregion Button Events

		#region CheckBox Events

		private void OnCheckedAlwaysOnTop(object sender, EventArgs e)
		{
			TopMost = ((CheckBox)sender).Checked;
		}

		private void OnCheckedAutoFace(object sender, EventArgs e)
		{
			GameSettingsManager.FaceTargetOnAction = cbxAutoFace.Checked;
		}

		#endregion CheckBox Events

		#region ComboBox Events

		private void OnSelectedCombatRoutine(object sender, EventArgs e)
		{
			string selected = (sender as ComboBox).Items[(sender as ComboBox).SelectedIndex].ToString();
			if (selected.Equals(RoutineManager.Current.Name)) return;
			RoutineManager.Current.ShutDown();
			RoutineManager.PreferedRoutine = (sender as ComboBox).Items[(sender as ComboBox).SelectedIndex].ToString();
			RoutineManager.PickRoutine();
		}

		private void OnSelectedHotkeyModifierAddTargetList(object sender, EventArgs e)
		{
			Settings.Default.HOTKEY_MODIFIER_ADD_TARGET_LIST = ((ComboBox)sender).SelectedIndex;
		}

		private void OnSelectedHotkeyModifierMovement(object sender, EventArgs e)
		{
			Settings.Default.HOTKEY_MODIFIER_TOGGLE_MOVEMENT = ((ComboBox)sender).SelectedIndex;
		}

		private void OnSelectedHotkeyModifierMovementMode(object sender, EventArgs e)
		{
			Settings.Default.HOTKEY_MODIFIER_MOVEMENT_MODE = ((ComboBox)sender).SelectedIndex;
		}

		private void OnSelectedHotkeyModifierPause(object sender, EventArgs e)
		{
			Settings.Default.HOTKEY_MODIFIER_PAUSE = ((ComboBox)sender).SelectedIndex;
		}

		private void OnSelectedHotkeyModifierRemoveTargetList(object sender, EventArgs e)
		{
			Settings.Default.HOTKEY_MODIFIER_REM_TARGET_LIST = ((ComboBox)sender).SelectedIndex;
		}

		private void OnSelectedHotkeyModifierTargetMode(object sender, EventArgs e)
		{
			Settings.Default.HOTKEY_MODIFIER_TARGET_MODE = ((ComboBox)sender).SelectedIndex;
		}

		private void OnSelectedMoveTarget(object sender, EventArgs e)
		{
			Settings.Default.SELECTED_MOVEMENT_MODE = ((ComboBox)sender).SelectedIndex;
		}

		private void OnSelectedNavigationProvider(object sender, EventArgs e)
		{
			Settings.Default.SELECTED_NAVIGATION_PROVIDER = ((ComboBox)sender).SelectedIndex;
			MudBase.UpdateNavigationProvider();
		}

		private void OnSelectedTargetingMode(object sender, EventArgs e)
		{
			Settings.Default.SELECTED_TARGETING_MODE = ((ComboBox)sender).SelectedIndex;
		}

		private void OnSelectedTargetListType(object sender, EventArgs e)
		{
			Settings.Default.SELECTED_TARGET_LIST_TYPE = ((ComboBox)sender).SelectedIndex;
		}

		#endregion ComboBox Events
	}
}